@@ -4,12 +4,15 @@ import numpy as np
 import pandas as pd
 from pandas import Series, DataFrame, MultiIndex
 from pandas.core.indexing import _NDFrameIndexer
-from shapely.geometry import box, MultiPoint, MultiLineString, MultiPolygon
+
+from shapely.geometry.base import BaseGeometry
+from shapely.geometry import box
 from shapely.ops import cascaded_union, unary_union
 import shapely.affinity as affinity
 
 import geopandas as gpd
 
+
 try:
     from rtree.core import RTreeError
     HAS_SINDEX = True
@@ -19,52 +22,89 @@ except ImportError:
     HAS_SINDEX = False
 
 
-def _geo_op(this, other, op):
-    """Operation that returns a GeoSeries"""
-    if isinstance(other, GeoPandasBase):
-        this = this.geometry
-        crs = this.crs
-        if crs != other.crs:
-            warn('GeoSeries crs mismatch: {0} and {1}'.format(this.crs,
-                                                              other.crs))
-        this, other = this.align(other.geometry)
-        return gpd.GeoSeries([getattr(this_elem, op)(other_elem)
-                             for this_elem, other_elem in zip(this, other)],
-                             index=this.index, crs=crs)
+def _binary_geo(op, left, right):
+    # type: (str, GeoSeries, GeoSeries) -> GeoSeries
+    """Binary operation on GeoSeries objects that returns a GeoSeries"""
+    from .geoseries import GeoSeries
+    if isinstance(right, GeoPandasBase):
+        left = left.geometry
+        left, right = left.align(right.geometry)
+
+        if left.crs != right.crs:
+            warn('GeoSeries crs mismatch: {0} and {1}'.format(left.crs,
+                                                              right.crs))
+
+        # intersection can return empty GeometryCollections, and if the result
+        # are only those, numpy will coerce it to empty 2D array
+        data = np.empty(len(left), dtype=object)
+        data[:] = [getattr(this_elem, op)(other_elem)
+                   for this_elem, other_elem in zip(left, right)]
+
+        return GeoSeries(data, index=left.index, crs=left.crs)
+
+    elif isinstance(right, BaseGeometry):
+        # ensure 1D output, see note above
+        data = np.empty(len(left), dtype=object)
+        data[:] = [getattr(s, op)(right) for s in left.geometry]
+        return GeoSeries(data, index=left.index, crs=left.crs)
     else:
-        return gpd.GeoSeries([getattr(s, op)(other)
-                             for s in this.geometry],
-                             index=this.index, crs=this.crs)
+        raise TypeError(type(left), type(right))
 
 
-# TODO: think about merging with _geo_op
-def _series_op(this, other, op, **kwargs):
-    """Geometric operation that returns a pandas Series"""
-    null_val = False if op not in ['distance', 'project'] else np.nan
+def _binary_op(op, this, other, *args, **kwargs):
+    # type: (str, GeoSeries, GeoSeries, args/kwargs) -> Series[bool]
+    """Binary operation on GeoSeries objects that returns a Series"""
+    if op in ['distance', 'project']:
+        null_value = np.nan
+    elif op == 'relate':
+        null_value = None
+    else:
+        null_value = False
+    if op in ['distance', 'project']:
+        dtype = float
+    elif op == 'relate':
+        dtype = object
+    else:
+        dtype = bool
 
     if isinstance(other, GeoPandasBase):
+
         this = this.geometry
         this, other = this.align(other.geometry)
-        return Series([getattr(this_elem, op)(other_elem, **kwargs)
-                    if not this_elem.is_empty | other_elem.is_empty else null_val
-                    for this_elem, other_elem in zip(this, other)],
-                    index=this.index, dtype=np.dtype(type(null_val)))
+
+        data = np.array(
+            [getattr(this_elem, op)(other_elem, *args, **kwargs)
+             if not this_elem.is_empty | other_elem.is_empty else null_value
+             for this_elem, other_elem in zip(this, other)],
+            dtype=dtype)
+
+        return Series(data, index=this.index)
+
+    elif isinstance(other, BaseGeometry):
+        data = np.array(
+            [getattr(s, op)(other, *args, **kwargs) if s else null_value
+             for s in this.geometry],
+            dtype=dtype)
+        return Series(data, index=this.index)
     else:
-        return Series([getattr(s, op)(other, **kwargs) if s else null_val
-                       for s in this.geometry],
-                      index=this.index, dtype=np.dtype(type(null_val)))
+        raise TypeError(type(this), type(other))
 
 
-def _geo_unary_op(this, op):
+def _unary_geo(op, this):
+    # type: (str, GeoSeries) -> GeoSeries
     """Unary operation that returns a GeoSeries"""
-    return gpd.GeoSeries([getattr(geom, op) for geom in this.geometry],
-                     index=this.index, crs=this.crs)
+    from .geoseries import GeoSeries
+    # ensure 1D output, see note above
+    data = np.empty(len(this), dtype=object)
+    data[:] = [getattr(geom, op) for geom in this.geometry]
+    return GeoSeries(data, index=this.index, crs=this.crs)
 
 
-def _series_unary_op(this, op, null_value=False):
+def _unary_op(op, this, null_value=False):
+    # type: (str, GeoSeries, Any) -> Series
     """Unary operation that returns a Series"""
     return Series([getattr(geom, op, null_value) for geom in this.geometry],
-                     index=this.index, dtype=np.dtype(type(null_value)))
+                  index=this.index, dtype=np.dtype(type(null_value)))
 
 
 class GeoPandasBase(object):
@@ -77,8 +117,8 @@ class GeoPandasBase(object):
         else:
             from geopandas.sindex import SpatialIndex
             stream = ((i, item.bounds, idx) for i, (idx, item) in
-                   enumerate(self.geometry.iteritems()) if
-                   pd.notnull(item) and not item.is_empty)
+                      enumerate(self.geometry.iteritems())
+                      if pd.notnull(item) and not item.is_empty)
             try:
                 self._sindex = SpatialIndex(stream)
             # What we really want here is an empty generator error, or
@@ -101,13 +141,13 @@ class GeoPandasBase(object):
     def area(self):
         """Returns a ``Series`` containing the area of each geometry in the
         ``GeoSeries``."""
-        return _series_unary_op(self, 'area', null_value=np.nan)
+        return _unary_op('area', self, null_value=np.nan)
 
     @property
     def geom_type(self):
         """Returns a ``Series`` of strings specifying the `Geometry Type` of each
         object."""
-        return _series_unary_op(self, 'geom_type', null_value=None)
+        return _unary_op('geom_type', self, null_value=None)
 
     @property
     def type(self):
@@ -117,19 +157,19 @@ class GeoPandasBase(object):
     @property
     def length(self):
         """Returns a ``Series`` containing the length of each geometry."""
-        return _series_unary_op(self, 'length', null_value=np.nan)
+        return _unary_op('length', self, null_value=np.nan)
 
     @property
     def is_valid(self):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
         geometries that are valid."""
-        return _series_unary_op(self, 'is_valid', null_value=False)
+        return _unary_op('is_valid', self, null_value=False)
 
     @property
     def is_empty(self):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
         empty geometries."""
-        return _series_unary_op(self, 'is_empty', null_value=False)
+        return _unary_op('is_empty', self, null_value=False)
 
     @property
     def is_simple(self):
@@ -138,13 +178,13 @@ class GeoPandasBase(object):
 
         This is meaningful only for `LineStrings` and `LinearRings`.
         """
-        return _series_unary_op(self, 'is_simple', null_value=False)
+        return _unary_op('is_simple', self, null_value=False)
 
     @property
     def is_ring(self):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
         features that are closed."""
-        # operates on the exterior, so can't use _series_unary_op()
+        # operates on the exterior, so can't use _unary_op()
         return Series([geom.exterior.is_ring for geom in self.geometry],
                       index=self.index)
 
@@ -152,8 +192,7 @@ class GeoPandasBase(object):
     def has_z(self):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
         features that have a z-component."""
-        # operates on the exterior, so can't use _series_unary_op()
-        return _series_unary_op(self, 'has_z', null_value=False)
+        return _unary_op('has_z', self, null_value=False)
 
     #
     # Unary operations that return a GeoSeries
@@ -163,13 +202,13 @@ class GeoPandasBase(object):
     def boundary(self):
         """Returns a ``GeoSeries`` of lower dimensional objects representing
         each geometries's set-theoretic `boundary`."""
-        return _geo_unary_op(self, 'boundary')
+        return _unary_geo('boundary', self)
 
     @property
     def centroid(self):
         """Returns a ``GeoSeries`` of points representing the centroid of each
         geometry."""
-        return _geo_unary_op(self, 'centroid')
+        return _unary_geo('centroid', self)
 
     @property
     def convex_hull(self):
@@ -180,7 +219,7 @@ class GeoPandasBase(object):
         containing all the points in each geometry, unless the number of points
         in the geometric object is less than three. For two points, the convex
         hull collapses to a `LineString`; for 1, a `Point`."""
-        return _geo_unary_op(self, 'convex_hull')
+        return _unary_geo('convex_hull', self)
 
     @property
     def envelope(self):
@@ -190,7 +229,7 @@ class GeoPandasBase(object):
         The envelope of a geometry is the bounding rectangle. That is, the
         point or smallest rectangular polygon (with sides parallel to the
         coordinate axes) that contains the geometry."""
-        return _geo_unary_op(self, 'envelope')
+        return _unary_geo('envelope', self)
 
     @property
     def exterior(self):
@@ -200,25 +239,48 @@ class GeoPandasBase(object):
         Applies to GeoSeries containing only Polygons.
         """
         # TODO: return empty geometry for non-polygons
-        return _geo_unary_op(self, 'exterior')
+        return _unary_geo('exterior', self)
 
     @property
     def interiors(self):
-        """Returns a ``GeoSeries`` of InteriorRingSequences representing the
+        """Returns a ``Series`` of List representing the
         inner rings of each polygon in the GeoSeries.
 
         Applies to GeoSeries containing only Polygons.
+
+        Returns
+        ----------
+        inner_rings: Series of List
+            Inner rings of each polygon in the GeoSeries.
         """
-        # TODO: return empty list or None for non-polygons
-        return _series_unary_op(self, 'interiors', null_value=False)
+
+        has_non_poly = False
+        inner_rings = []
+        for geom in self.geometry:
+            interior_ring_seq = getattr(geom, 'interiors', None)
+            # polygon case
+            if interior_ring_seq is not None:
+                inner_rings.append(list(interior_ring_seq))
+            # non-polygon case
+            else:
+                has_non_poly = True
+                inner_rings.append(None)
+        if has_non_poly:
+            warn("Only Polygon objects have interior rings. For other "
+                 "geometry types, None is returned.")
+
+        # _unary_op couldn't be used in order to warning to non-polygon and
+        # conversion to list.
+        return Series(inner_rings,
+                      index=self.index, dtype=object)
 
     def representative_point(self):
         """Returns a ``GeoSeries`` of (cheaply computed) points that are
         guaranteed to be within each geometry.
         """
         return gpd.GeoSeries([geom.representative_point()
-                             for geom in self.geometry],
-                         index=self.index)
+                              for geom in self.geometry],
+                             index=self.index)
 
     #
     # Reduction operations that return a Shapely geometry
@@ -256,7 +318,7 @@ class GeoPandasBase(object):
             The GeoSeries (elementwise) or geometric object to test if is
             contained.
         """
-        return _series_op(self, other, 'contains')
+        return _binary_op('contains', self, other)
 
     def geom_equals(self, other):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
@@ -272,7 +334,7 @@ class GeoPandasBase(object):
             The GeoSeries (elementwise) or geometric object to test for
             equality.
         """
-        return _series_op(self, other, 'equals')
+        return _binary_op('equals', self, other)
 
     def geom_almost_equals(self, other, decimal=6):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` if
@@ -288,14 +350,12 @@ class GeoPandasBase(object):
         decimal : int
             Decimal place presion used when testing for approximate equality.
         """
-        # TODO: pass precision argument
-        return _series_op(self, other, 'almost_equals', decimal=decimal)
+        return _binary_op('almost_equals', self, other, decimal=decimal)
 
     def geom_equals_exact(self, other, tolerance):
         """Return True for all geometries that equal *other* to a given
         tolerance, else False"""
-        # TODO: pass tolerance argument.
-        return _series_op(self, other, 'equals_exact', tolerance=tolerance)
+        return _binary_op('equals_exact', self, other, tolerance=tolerance)
 
     def crosses(self, other):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
@@ -311,7 +371,7 @@ class GeoPandasBase(object):
             The GeoSeries (elementwise) or geometric object to test if is
             crossed.
         """
-        return _series_op(self, other, 'crosses')
+        return _binary_op('crosses', self, other)
 
     def disjoint(self, other):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
@@ -326,7 +386,7 @@ class GeoPandasBase(object):
             The GeoSeries (elementwise) or geometric object to test if is
             disjoint.
         """
-        return _series_op(self, other, 'disjoint')
+        return _binary_op('disjoint', self, other)
 
     def intersects(self, other):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
@@ -341,11 +401,11 @@ class GeoPandasBase(object):
             The GeoSeries (elementwise) or geometric object to test if is
             intersected.
         """
-        return _series_op(self, other, 'intersects')
+        return _binary_op('intersects', self, other)
 
     def overlaps(self, other):
         """Return True for all geometries that overlap *other*, else False"""
-        return _series_op(self, other, 'overlaps')
+        return _binary_op('overlaps', self, other)
 
     def touches(self, other):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
@@ -361,7 +421,7 @@ class GeoPandasBase(object):
             The GeoSeries (elementwise) or geometric object to test if is
             touched.
         """
-        return _series_op(self, other, 'touches')
+        return _binary_op('touches', self, other)
 
     def within(self, other):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
@@ -382,7 +442,7 @@ class GeoPandasBase(object):
             geometry is within.
 
         """
-        return _series_op(self, other, 'within')
+        return _binary_op('within', self, other)
 
     def distance(self, other):
         """Returns a ``Series`` containing the distance to `other`.
@@ -393,7 +453,7 @@ class GeoPandasBase(object):
             The Geoseries (elementwise) or geometric object to find the
             distance to.
         """
-        return _series_op(self, other, 'distance')
+        return _binary_op('distance', self, other)
 
     #
     # Binary operations that return a GeoSeries
@@ -409,7 +469,7 @@ class GeoPandasBase(object):
             The Geoseries (elementwise) or geometric object to find the
             difference to.
         """
-        return _geo_op(self, other, 'difference')
+        return _binary_geo('difference', self, other)
 
     def symmetric_difference(self, other):
         """Returns a ``GeoSeries`` of the symmetric difference of points in
@@ -424,7 +484,7 @@ class GeoPandasBase(object):
             The Geoseries (elementwise) or geometric object to find the
             symmetric difference to.
         """
-        return _geo_op(self, other, 'symmetric_difference')
+        return _binary_geo('symmetric_difference', self, other)
 
     def union(self, other):
         """Returns a ``GeoSeries`` of the union of points in each geometry with
@@ -436,7 +496,7 @@ class GeoPandasBase(object):
             The Geoseries (elementwise) or geometric object to find the union
             with.
         """
-        return _geo_op(self, other, 'union')
+        return _binary_geo('union', self, other,)
 
     def intersection(self, other):
         """Returns a ``GeoSeries`` of the intersection of points in each
@@ -448,7 +508,7 @@ class GeoPandasBase(object):
             The Geoseries (elementwise) or geometric object to find the
             intersection with.
         """
-        return _geo_op(self, other, 'intersection')
+        return _binary_geo('intersection', self, other)
 
     #
     # Other operations
@@ -495,14 +555,28 @@ class GeoPandasBase(object):
 
         Parameters
         ----------
-        distance : float
-            The radius of the buffer.
-        resolution: float
+        distance : float, np.array, pd.Series
+            The radius of the buffer. If np.array or pd.Series are used
+            then it must have same length as the GeoSeries.
+        resolution: int
             Optional, the resolution of the buffer around each vertex.
         """
+        if isinstance(distance, (np.ndarray, pd.Series)):
+            if len(distance) != len(self.index):
+                raise ValueError("Length of distance sequence does not match "
+                                 "length of the GeoSeries")
+            if isinstance(distance, pd.Series):
+                if not self.index.equals(distance.index):
+                    raise ValueError("Index values of distance sequence does "
+                                     "not match index values of the GeoSeries")
+            return gpd.GeoSeries(
+                [geom.buffer(dist, resolution, **kwargs)
+                 for geom, dist in zip(self.geometry, distance)],
+                index=self.index, crs=self.crs)
+
         return gpd.GeoSeries([geom.buffer(distance, resolution, **kwargs)
-                             for geom in self.geometry],
-                         index=self.index, crs=self.crs)
+                              for geom in self.geometry],
+                             index=self.index, crs=self.crs)
 
     def simplify(self, *args, **kwargs):
         """Returns a ``GeoSeries`` containing a simplified representation of
@@ -520,12 +594,27 @@ class GeoPandasBase(object):
             False uses a quicker algorithm, but may produce self-intersecting
             or otherwise invalid geometries.
         """
-        return gpd.GeoSeries([geom.simplify(*args, **kwargs)
-                             for geom in self.geometry],
-                      index=self.index, crs=self.crs)
+        return gpd.GeoSeries(
+            [geom.simplify(*args, **kwargs) for geom in self.geometry],
+            index=self.index, crs=self.crs)
 
     def relate(self, other):
-        raise NotImplementedError
+        """
+        Returns the DE-9IM intersection matrices for the geometries
+
+        Parameters
+        ----------
+        other : BaseGeometry or GeoSeries
+            The other geometry to computed
+            the DE-9IM intersection matrices from.
+
+        Returns
+        ----------
+        spatial_relations: Series of strings
+            The DE-9IM intersection matrices which describe
+            the spatial relations of the other geometry.
+        """
+        return _binary_op('relate', self, other)
 
     def project(self, other, normalized=False):
         """
@@ -542,7 +631,7 @@ class GeoPandasBase(object):
         The project method is the inverse of interpolate.
         """
 
-        return _series_op(self, other, 'project', normalized=normalized)
+        return _binary_op('project', self, other, normalized=normalized)
 
     def interpolate(self, distance, normalized=False):
         """
@@ -551,15 +640,29 @@ class GeoPandasBase(object):
         Parameters
         ----------
         distance : float or Series of floats
-            Distance(s) along the geometries at which a point should be returned
+            Distance(s) along the geometries at which a point should be
+            returned. If np.array or pd.Series are used then it must have
+            same length as the GeoSeries.
         normalized : boolean
             If normalized is True, distance will be interpreted as a fraction
             of the geometric object's length.
         """
+        if isinstance(distance, (np.ndarray, pd.Series)):
+            if len(distance) != len(self.index):
+                raise ValueError("Length of distance sequence does not match "
+                                 "length of the GeoSeries")
+            if isinstance(distance, pd.Series):
+                if not self.index.equals(distance.index):
+                    raise ValueError("Index values of distance sequence does "
+                                     "not match index values of the GeoSeries")
+            return gpd.GeoSeries(
+                [s.interpolate(dist, normalized=normalized)
+                 for (s, dist) in zip(self.geometry, distance)],
+                index=self.index, crs=self.crs)
 
-        return gpd.GeoSeries([s.interpolate(distance, normalized)
-                             for s in self.geometry],
-            index=self.index, crs=self.crs)
+        return gpd.GeoSeries([s.interpolate(distance, normalized=normalized)
+                              for s in self.geometry],
+                             index=self.index, crs=self.crs)
 
     def translate(self, xoff=0.0, yoff=0.0, zoff=0.0):
         """Returns a ``GeoSeries`` with translated geometries.
@@ -575,8 +678,8 @@ class GeoPandasBase(object):
             dimensions respectively.
         """
         return gpd.GeoSeries([affinity.translate(s, xoff, yoff, zoff)
-                             for s in self.geometry],
-            index=self.index, crs=self.crs)
+                              for s in self.geometry],
+                             index=self.index, crs=self.crs)
 
     def rotate(self, angle, origin='center', use_radians=False):
         """Returns a ``GeoSeries`` with rotated geometries.
@@ -597,8 +700,10 @@ class GeoPandasBase(object):
         use_radians : boolean
             Whether to interpret the angle of rotation as degrees or radians
         """
-        return gpd.GeoSeries([affinity.rotate(s, angle, origin=origin,
-            use_radians=use_radians) for s in self.geometry],
+        return gpd.GeoSeries(
+            [affinity.rotate(s, angle, origin=origin,
+                             use_radians=use_radians)
+             for s in self.geometry],
             index=self.index, crs=self.crs)
 
     def scale(self, xfact=1.0, yfact=1.0, zfact=1.0, origin='center'):
@@ -619,9 +724,10 @@ class GeoPandasBase(object):
             box center (default), 'centroid' for the geometry's 2D centroid, a
             Point object or a coordinate tuple (x, y, z).
         """
-        return gpd.GeoSeries([affinity.scale(s, xfact, yfact, zfact,
-            origin=origin) for s in self.geometry], index=self.index,
-            crs=self.crs)
+        return gpd.GeoSeries(
+            [affinity.scale(s, xfact, yfact, zfact, origin=origin)
+             for s in self.geometry],
+            index=self.index, crs=self.crs)
 
     def skew(self, xs=0.0, ys=0.0, origin='center', use_radians=False):
         """Returns a ``GeoSeries`` with skewed geometries.
@@ -644,8 +750,9 @@ class GeoPandasBase(object):
         use_radians : boolean
             Whether to interpret the shear angle(s) as degrees or radians
         """
-        return gpd.GeoSeries([affinity.skew(s, xs, ys, origin=origin,
-            use_radians=use_radians) for s in self.geometry],
+        return gpd.GeoSeries(
+            [affinity.skew(s, xs, ys, origin=origin, use_radians=use_radians)
+             for s in self.geometry],
             index=self.index, crs=self.crs)
 
     def explode(self):
@@ -659,8 +766,8 @@ class GeoPandasBase(object):
         Returns
         ------
         A GeoSeries with a MultiIndex. The levels of the MultiIndex are the
-        original index and a zero-based integer index that counts the 
-        number of single geometries within a multi-part geometry. 
+        original index and a zero-based integer index that counts the
+        number of single geometries within a multi-part geometry.
 
         Example
         -------
