@@ -64,7 +64,6 @@ class TestGeomMethods:
         self.g5 = GeoSeries([self.l1, self.l2])
         self.g6 = GeoSeries([self.p0, self.t3])
         self.empty = GeoSeries([])
-        self.empty.crs = {'init': 'epsg:4326', 'no_defs': True}
         self.empty_poly = Polygon()
 
         # Crossed lines
@@ -238,6 +237,20 @@ class TestGeomMethods:
         self._test_binary_topological('difference', expected,
                                       self.g1, self.t2)
 
+    def test_geo_op_empty_result(self):
+        l1 = LineString([(0, 0), (1, 1)])
+        l2 = LineString([(2, 2), (3, 3)])
+        expected = GeoSeries([GeometryCollection()])
+        # binary geo resulting in empty geometry
+        result = GeoSeries([l1]).intersection(l2)
+        assert_geoseries_equal(result, expected)
+        # binary geo empty result with right GeoSeries
+        result = GeoSeries([l1]).intersection(GeoSeries([l2]))
+        assert_geoseries_equal(result, expected)
+        # unary geo resulting in emtpy geometry
+        result = GeoSeries([GeometryCollection()]).convex_hull
+        assert_geoseries_equal(result, expected)
+
     def test_boundary(self):
         l1 = LineString([(0, 0), (1, 0), (1, 1), (0, 0)])
         l2 = LineString([(0, 0), (1, 0), (1, 1), (0, 1), (0, 0)])
@@ -298,6 +311,21 @@ class TestGeomMethods:
         expected = [False, False, False, False, False, True]
         assert_array_dtype_equal(expected, self.g0.disjoint(self.t1))
 
+    def test_relate(self):
+        expected = Series(['212101212',
+                           '212101212',
+                           '212FF1FF2',
+                           '2FFF1FFF2',
+                           'FF2F112F2',
+                           'FF0FFF212'],
+                          index=self.g0.index)
+        assert_array_dtype_equal(expected, self.g0.relate(self.inner_sq))
+
+        expected = Series(['FF0FFF212',
+                           None],
+                          index=self.g6.index)
+        assert_array_dtype_equal(expected, self.g6.relate(self.na_none))
+
     def test_distance(self):
         expected = Series(np.array([np.sqrt((5 - 1)**2 + (5 - 1)**2), np.nan]),
                           self.na_none.index)
@@ -395,10 +423,14 @@ class TestGeomMethods:
             assert computed.equals(expected)
 
     def test_interiors(self):
-        square_series = GeoSeries(self.nested_squares)
-        exp_interiors = GeoSeries([LinearRing(self.inner_sq.boundary)])
-        for expected, computed in zip(exp_interiors, square_series.interiors):
-            assert computed[0].equals(expected)
+        original = GeoSeries([self.t1, self.nested_squares])
+
+        # This is a polygon with no interior.
+        expected = []
+        assert original.interiors[0] == expected
+        # This is a polygon with an interior.
+        expected = LinearRing(self.inner_sq.boundary)
+        assert original.interiors[1][0].equals(expected)
 
     def test_interpolate(self):
         expected = GeoSeries([Point(0.5, 1.0), Point(0.75, 1.0)])
@@ -409,6 +441,25 @@ class TestGeomMethods:
         self._test_binary_topological('interpolate', expected, self.g5,
                                       1.5)
 
+    def test_interpolate_distance_array(self):
+        expected = GeoSeries([Point(0.0, 0.75), Point(1.0, 0.5)])
+        self._test_binary_topological('interpolate', expected, self.g5,
+                                      np.array([0.75, 1.5]))
+
+        expected = GeoSeries([Point(0.5, 1.0), Point(0.0, 1.0)])
+        self._test_binary_topological('interpolate', expected, self.g5,
+                                      np.array([0.75, 1.5]), normalized=True)
+
+    def test_interpolate_distance_wrong_length(self):
+        distances = np.array([1, 2, 3])
+        with pytest.raises(ValueError):
+            self.g5.interpolate(distances)
+
+    def test_interpolate_distance_wrong_index(self):
+        distances = Series([1, 2], index=[99, 98])
+        with pytest.raises(ValueError):
+            self.g5.interpolate(distances)
+
     def test_project(self):
         expected = Series([2.0, 1.5], index=self.g5.index)
         p = Point(1.0, 0.5)
@@ -486,6 +537,27 @@ class TestGeomMethods:
             expected = original.buffer(10, **args)
             assert calculated.equals(expected)
 
+    def test_buffer_distance_array(self):
+        original = GeoSeries([self.p0, self.p0])
+        expected = GeoSeries(
+            [Polygon(((6, 5), (5, 4), (4, 5), (5, 6), (6, 5))),
+             Polygon(((10, 5), (5, 0), (0, 5), (5, 10), (10, 5))),
+             ])
+        calculated = original.buffer(np.array([1, 5]), resolution=1)
+        assert_geoseries_equal(calculated, expected, check_less_precise=True)
+
+    def test_buffer_distance_wrong_length(self):
+        original = GeoSeries([self.p0, self.p0])
+        distances = np.array([1, 2, 3])
+        with pytest.raises(ValueError):
+            original.buffer(distances)
+
+    def test_buffer_distance_wrong_index(self):
+        original = GeoSeries([self.p0, self.p0], index=[0, 1])
+        distances = Series(data=[1, 2], index=[99, 98])
+        with pytest.raises(ValueError):
+            original.buffer(distances)
+
     def test_envelope(self):
         e = self.g3.envelope
         assert np.all(e.geom_equals(self.sq))
@@ -523,8 +595,8 @@ class TestGeomMethods:
 
         expected_s = GeoSeries([Point(1, 2), Point(2, 3), Point(5, 5)])
         expected_df = GeoDataFrame({'col': [1, 1, 2], 'geometry': expected_s})
-        expected_index = MultiIndex(levels=[[0, 1], [0, 1]],
-                                    labels=[[0, 0, 1], [0, 1, 0]],
+        expected_index = MultiIndex([[0, 1], [0, 1]],  # levels
+                                    [[0, 0, 1], [0, 1, 0]],  # labels/codes
                                     names=[index_name, None])
         expected_df = expected_df.set_index(expected_index)
         assert_frame_equal(test_df, expected_df)
