@@ -1,5 +1,7 @@
 from __future__ import absolute_import
 
+import warnings
+
 from pandas import Series, DataFrame
 from shapely.geometry import Point
 
@@ -41,9 +43,6 @@ class TestSeries:
     def test_take(self):
         assert type(self.pts.take(list(range(0, self.N, 2)))) is GeoSeries
 
-    def test_select(self):
-        assert type(self.pts.select(lambda x: x % 2 == 0)) is GeoSeries
-
     def test_groupby(self):
         for f, s in self.pts.groupby(lambda x: x % 2):
             assert type(s) is GeoSeries
